exports.postgres = (engine) ->

  extensions = []
  overrides = []

  # Query API Extensions
  engine.extendQuery = (query) ->
      
    # Add support for common table expressions
    extensions.push('with')
    query.with = (name, cte) ->
      ###
      Usage (example from http://stackoverflow.com/a/3800572/446634)::
      
        select('summary', ['*']).where(rk: 1)
          .with('summary', select {p: 'purchases'}, ['id', 'customer', 'total'], ->
            @fields func('row_number')
            @over partition: ['customer'], order: [{rk: 'total DESC'}])

        WITH summary AS (
            SELECT p.id, p.customer, p.total, ROW_NUMBER()
              OVER(PARTITION BY p.customer ORDER BY p.total DESC) AS rk
              FROM PURCHASES p)
        SELECT s.*
          FROM summary s
         WHERE s.rk = 1
      ###
      (@ctes ?= {})[name] = cte

    overrides.push('render')
    query.render = (args...) ->
      return @_override.render.apply(@, args) unless @ctes
      "WITH " + ("#{alias} AS (#{q.render(args...)}" for alias, q of @ctes).join("\n") + "\n"
      @_override.render.apply(@, args)

    overrides.push('compile')
    query.compile = ->
      return  @_override.compile.apply(@, arguments) unless @ctes
      params = @q.params()
      for _, cte of @ctes
        params = cte.q.params().concat(params)
      [@render(), params]

  engine.unextendQuery = (query) ->
    delete query[name] for name in extensions
    query[name] = query.constructor::[name] for name in overrides
